@@ -174,6 +174,7 @@ pub fn functions() -> HashSet<&'static str>
         "is_prime",
         "isprime",
         "dice",
+        "μ",
         "W",
         "productlog",
         "lambertw",
@@ -201,6 +202,8 @@ pub fn functions() -> HashSet<&'static str>
         "trigamma",
         "pochhammer",
         "ph",
+        "ψ",
+        "next",
     ]
     .iter()
     .cloned()
