@@ -3,12 +3,15 @@ use crate::{
         NumStr,
         NumStr::{Matrix, Num, Str, Vector},
     },
+    load_vars::set_commands_or_vars,
     math::do_math,
     misc::{
         get_terminal_dimensions_pixel, place_funcvar, place_funcvarxy, place_var, place_varxy,
         prompt,
     },
-    Colors, Options,
+    options::silent_commands,
+    parse::input_var,
+    Colors, Options, Variable,
 };
 use gnuplot::{Auto, AxesCommon, Caption, Color, Figure, Fix, PointSymbol, TickOption};
 use rug::Complex;
@@ -20,19 +23,97 @@ use std::{
 };
 #[allow(clippy::type_complexity)]
 pub fn graph(
-    input: Vec<String>,
-    func: Vec<(Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options)>,
+    mut input: Vec<String>,
+    mut vars: Vec<Variable>,
+    mut options: Options,
     watch: Option<Instant>,
-    colors: Colors,
+    mut colors: Colors,
+    cli: bool,
 ) -> JoinHandle<()>
 {
     thread::spawn(move || {
+        let mut func = Vec::new();
+        for (i, s) in input.clone().iter().enumerate()
+        {
+            if s.is_empty()
+            {
+                continue;
+            }
+            {
+                options.prec = options.graph_prec;
+                let split = s.split(|c| c == ';');
+                let count = split.clone().count();
+                if count != 1
+                {
+                    input[i] = split.clone().last().unwrap().to_string();
+                    for (i, s) in split.enumerate()
+                    {
+                        if i == count - 1
+                        {
+                            break;
+                        }
+                        silent_commands(
+                            &mut options,
+                            &s.chars()
+                                .filter(|c| !c.is_whitespace())
+                                .collect::<Vec<char>>(),
+                        );
+                        if s.contains('=')
+                        {
+                            if let Err(s) = set_commands_or_vars(
+                                &mut colors,
+                                &mut options,
+                                &mut vars,
+                                &s.chars().collect::<Vec<char>>(),
+                            )
+                            {
+                                print!(
+                                    "\x1b[G\x1b[A\x1b[K{}\n\x1b[G{}",
+                                    s,
+                                    prompt(options, &colors)
+                                );
+                                stdout().flush().unwrap()
+                            }
+                        }
+                    }
+                }
+            }
+            options.prec = options.graph_prec;
+            func.push(
+                match input_var(
+                    &input[i],
+                    vars.clone(),
+                    &mut Vec::new(),
+                    &mut 0,
+                    options,
+                    false,
+                    false,
+                    0,
+                    Vec::new(),
+                )
+                {
+                    Ok(f) => (f.0, f.1, options),
+                    _ =>
+                    {
+                        print!("\x1b[G\x1b[Kbad input\x1b[G\n{}", prompt(options, &colors));
+                        stdout().flush().unwrap();
+                        return;
+                    }
+                },
+            );
+        }
         if input.iter().all(|i| i.is_empty())
         {
             return;
         }
         let mut options = func[0].2;
         let mut fg = Figure::new();
+        if cli
+        {
+            options.onaxis = false;
+            options.scale_graph = false;
+            fg.set_terminal("dumb size 125,60 aspect 1,1", "");
+        }
         fg.set_enhanced_text(false);
         let mut re_cap: [String; 6] = Default::default();
         let mut im_cap: [String; 6] = Default::default();
@@ -58,7 +139,7 @@ pub fn graph(
             {
                 return;
             }
-            if re_or_im.0
+            if re_or_im.0 || !re_or_im.1
             {
                 re_cap[i] = input[i].clone() + if re_or_im.1 { ":re" } else { "" }
             }
@@ -175,16 +256,18 @@ pub fn graph(
             {
                 Auto
             };
-            if options.lines || lines
+            if (options.lines || lines) && lines ^ options.lines
             {
                 fg.axes2d()
-                    .set_x_grid(true)
-                    .set_y_grid(true)
+                    .set_x_grid(!cli)
+                    .set_y_grid(!cli)
                     .set_aspect_ratio(ratio)
                     .set_x_ticks(xticks, &[TickOption::OnAxis(options.onaxis)], &[])
                     .set_y_ticks(yticks, &[TickOption::OnAxis(options.onaxis)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
+                    .set_x_label(if options.depth { "re" } else { &colors.label.0 }, &[])
+                    .set_y_label(if options.depth { "im" } else { &colors.label.1 }, &[])
                     .lines([0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -303,13 +386,15 @@ pub fn graph(
             else
             {
                 fg.axes2d()
-                    .set_x_grid(true)
-                    .set_y_grid(true)
+                    .set_x_grid(!cli)
+                    .set_y_grid(!cli)
                     .set_aspect_ratio(ratio)
                     .set_x_ticks(xticks, &[TickOption::OnAxis(options.onaxis)], &[])
                     .set_y_ticks(yticks, &[TickOption::OnAxis(options.onaxis)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
+                    .set_x_label(if options.depth { "re" } else { &colors.label.0 }, &[])
+                    .set_y_label(if options.depth { "im" } else { &colors.label.1 }, &[])
                     .lines([0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -537,18 +622,19 @@ pub fn graph(
                 && (points3d[0][0][2].is_empty() || points3d[0][1][2].is_empty())
             {
                 fg.axes3d()
-                    .set_x_grid(true)
-                    .set_y_grid(true)
-                    .set_z_grid(true)
-                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis)], &[])
-                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis)], &[])
-                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    //.set_palette(Custom(&[(0.0, 0.0, 0.0, 0.0), (1.0, 1.0, 1.0, 1.0)]))
+                    .set_x_grid(!cli)
+                    .set_y_grid(!cli)
+                    .set_z_grid(!cli)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
+                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
                     .set_z_range(Fix(options.zr.0), Fix(options.zr.1))
-                    .set_x_label("x", &[])
-                    .set_y_label(if options.depth { "re" } else { "y" }, &[])
-                    .set_z_label(if options.depth { "im" } else { "z" }, &[])
+                    .set_x_label(&colors.label.0, &[])
+                    .set_y_label(if options.depth { "re" } else { &colors.label.1 }, &[])
+                    .set_z_label(if options.depth { "im" } else { &colors.label.2 }, &[])
                     .lines([0], [0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .surface(
@@ -569,18 +655,18 @@ pub fn graph(
             else if options.lines || lines
             {
                 fg.axes3d()
-                    .set_x_grid(true)
-                    .set_y_grid(true)
-                    .set_z_grid(true)
-                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis)], &[])
-                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis)], &[])
-                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_x_grid(!cli)
+                    .set_y_grid(!cli)
+                    .set_z_grid(!cli)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
+                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
                     .set_z_range(Fix(options.zr.0), Fix(options.zr.1))
-                    .set_x_label("x", &[])
-                    .set_y_label(if options.depth { "re" } else { "y" }, &[])
-                    .set_z_label(if options.depth { "im" } else { "z" }, &[])
+                    .set_x_label(&colors.label.0, &[])
+                    .set_y_label(if options.depth { "re" } else { &colors.label.1 }, &[])
+                    .set_z_label(if options.depth { "im" } else { &colors.label.2 }, &[])
                     .lines([0], [0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -753,18 +839,18 @@ pub fn graph(
             else
             {
                 fg.axes3d()
-                    .set_x_grid(true)
-                    .set_y_grid(true)
-                    .set_z_grid(true)
-                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis)], &[])
-                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis)], &[])
-                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis)], &[])
+                    .set_x_grid(!cli)
+                    .set_y_grid(!cli)
+                    .set_z_grid(!cli)
+                    .set_x_ticks(xticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
+                    .set_y_ticks(yticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
+                    .set_z_ticks(zticks, &[TickOption::OnAxis(!options.onaxis && !cli)], &[])
                     .set_y_range(Fix(options.yr.0), Fix(options.yr.1))
                     .set_x_range(Fix(options.xr.0), Fix(options.xr.1))
                     .set_z_range(Fix(options.zr.0), Fix(options.zr.1))
-                    .set_x_label("x", &[])
-                    .set_y_label(if options.depth { "re" } else { "y" }, &[])
-                    .set_z_label(if options.depth { "im" } else { "z" }, &[])
+                    .set_x_label(&colors.label.0, &[])
+                    .set_y_label(if options.depth { "re" } else { &colors.label.1 }, &[])
+                    .set_z_label(if options.depth { "im" } else { &colors.label.2 }, &[])
                     .lines([0], [0], [0], &[Caption(&re_cap[0]), Color(&colors.re1col)])
                     .lines([0], [0], [0], &[Caption(&im_cap[0]), Color(&colors.im1col)])
                     .lines([0], [0], [0], &[Caption(&re_cap[1]), Color(&colors.re2col)])
@@ -1706,7 +1792,7 @@ fn get_data(
                         {
                             d2_or_d3.0 = true;
                             let mut vec = Vec::with_capacity(v.len());
-                            for i in 0..v.len()
+                            for i in 1..=v.len()
                             {
                                 vec.push(i as f64);
                             }
