@@ -17,14 +17,18 @@ use crate::{
     load_vars::{add_var, get_cli_vars, get_file_vars, get_vars, set_commands_or_vars},
     math::do_math,
     misc::{
-        clearln, convert, get_terminal_dimensions, handle_err, prompt, read_single_char, write,
+        clear, clearln, convert, get_terminal_dimensions, handle_err, prompt, read_single_char,
+        write,
     },
-    options::{arg_opts, commands, file_opts, silent_commands},
+    options::{arg_opts, commands, file_opts},
     parse::input_var,
     print::{print_answer, print_concurrent},
     AngleType::Radians,
 };
-use crossterm::terminal;
+use crossterm::{
+    cursor::{DisableBlinking, EnableBlinking},
+    execute, terminal,
+};
 use std::{
     env::{args, var},
     fs::{File, OpenOptions},
@@ -37,8 +41,11 @@ use std::{
 //rpn
 //matrix exponentiation
 //do as much as you can before graphing, make '=' at end show unsimplified, and no = show simplififed
-//stop vectors or matrixes disipearing graphically
-//maybe nth area
+//stop vectors or matrixes disipearing graphically by making them have points instead of lines
+//maybe nth area, make -nth go into the other function
+//allow setting x and y window dimensions of gnuplot, defaulting to what happens now
+//setting to disable default vars by default, in config
+//domain coloring
 //support units properly, add a part to the Num struct where it just stores the unit which then can be dealt with in complex or smth
 #[derive(Clone)]
 pub struct Variable
@@ -68,12 +75,13 @@ pub struct Colors
     im4col: String,
     im5col: String,
     im6col: String,
+    label: (String, String, String),
 }
 impl Default for Colors
 {
     fn default() -> Self
     {
-        Colors {
+        Self {
             text: "\x1b[0m".to_string(),
             prompt: "\x1b[94m".to_string(),
             imag: "\x1b[93m".to_string(),
@@ -98,6 +106,7 @@ impl Default for Colors
             im4col: "#00aa00".to_string(),
             im5col: "#00aaaa".to_string(),
             im6col: "#aaaa00".to_string(),
+            label: ("x".to_string(), "y".to_string(), "z".to_string()),
         }
     }
 }
@@ -150,12 +159,14 @@ pub struct Options
     interactive: bool,
     surface: bool,
     scale_graph: bool,
+    stay_interactive: bool,
+    graph_cli: bool,
 }
 impl Default for Options
 {
     fn default() -> Self
     {
-        Options {
+        Self {
             sci: false,
             deg: Radians,
             base: 10,
@@ -170,7 +181,7 @@ impl Default for Options
             prompt: true,
             comma: false,
             prec: (512, 512),
-            graph_prec: (64, 64),
+            graph_prec: (128, 128),
             frac_iter: 50,
             xr: (-10.0, 10.0),
             yr: (-10.0, 10.0),
@@ -178,8 +189,8 @@ impl Default for Options
             vxr: (0.0, 0.0),
             vyr: (0.0, 0.0),
             vzr: (0.0, 0.0),
-            samples_2d: 20000,
-            samples_3d: (300, 300),
+            samples_2d: 10000,
+            samples_3d: (250, 250),
             point_style: '.',
             lines: false,
             multi: true,
@@ -195,6 +206,8 @@ impl Default for Options
             interactive: true,
             surface: false,
             scale_graph: true,
+            stay_interactive: false,
+            graph_cli: false,
         }
     }
 }
@@ -224,6 +237,7 @@ fn main()
     }
     if !stdin().is_terminal()
     {
+        let mut lines = Vec::new();
         for line in stdin().lock().lines()
         {
             if !line.as_ref().unwrap().is_empty()
@@ -232,11 +246,12 @@ fn main()
                 {
                     if !line.starts_with('#') && !line.is_empty()
                     {
-                        args.push(line);
+                        lines.push(line);
                     }
                 }
             }
         }
+        args.splice(0..0, lines);
     }
     options.interactive = args.is_empty();
     let mut stdout = stdout();
@@ -266,7 +281,8 @@ fn main()
         "C:\\Users\\{}\\AppData\\Roaming\\kalc.vars",
         var("USERNAME").unwrap()
     );
-    let mut vars: Vec<Variable> = if options.allow_vars && options.interactive
+    let mut vars: Vec<Variable> = if options.allow_vars
+        && (options.interactive || options.stay_interactive)
     {
         get_vars(options)
     }
@@ -274,7 +290,7 @@ fn main()
     {
         Vec::new()
     };
-    if !options.interactive && options.allow_vars
+    if !options.interactive && options.allow_vars && !options.stay_interactive
     {
         get_cli_vars(options, args.concat(), &mut vars)
     }
@@ -299,7 +315,7 @@ fn main()
                     .collect::<Vec<String>>();
                 let mut split;
                 let args = args.concat();
-                let mut blacklist = if options.interactive
+                let mut blacklist = if options.interactive || options.stay_interactive
                 {
                     Vec::new()
                 }
@@ -322,12 +338,14 @@ fn main()
                         l.clone()
                     };
                     if options.interactive
+                        || options.stay_interactive
                         || (!blacklist.contains(&l)
                             && if l.contains('(')
                             {
                                 args.contains(&(left.clone() + "("))
                                     || args.contains(&(left.clone() + "{"))
                                     || args.contains(&(left.clone() + "["))
+                                    || args.contains(&(left.clone() + "|"))
                             }
                             else
                             {
@@ -337,7 +355,7 @@ fn main()
                         if let Some(r) = split.next()
                         {
                             let le = l.chars().collect::<Vec<char>>();
-                            if !options.interactive
+                            if !options.interactive && !options.stay_interactive
                             {
                                 get_file_vars(options, &mut vars, lines.clone(), r, &mut blacklist);
                                 if blacklist.contains(&l)
@@ -393,7 +411,7 @@ fn main()
             }
         }
     }
-    let (mut file, mut unmod_lines) = if options.interactive
+    let (mut file, mut unmod_lines) = if options.interactive || options.stay_interactive
     {
         #[cfg(unix)]
         let file_path = &(var("HOME").unwrap() + "/.config/kalc.history");
@@ -405,12 +423,6 @@ fn main()
         if File::open(file_path).is_err()
         {
             File::create(file_path).unwrap();
-            OpenOptions::new()
-                .append(true)
-                .open(file_path)
-                .unwrap()
-                .write_all(b"\n")
-                .unwrap();
         }
         (
             Some(OpenOptions::new().append(true).open(file_path).unwrap()),
@@ -429,7 +441,7 @@ fn main()
     };
     let mut handles: Vec<JoinHandle<()>> = Vec::new();
     let mut cut: Vec<char> = Vec::new();
-    'main: loop
+    loop
     {
         let mut input = Vec::new();
         let mut graphable = false;
@@ -496,6 +508,17 @@ fn main()
                     }
                 }
             }
+            if args.is_empty() && options.stay_interactive
+            {
+                options.interactive = true;
+                terminal::enable_raw_mode().unwrap();
+                print!(
+                    "\x1b[G\x1b[K{}{}",
+                    prompt(options, &colors),
+                    if options.color { "\x1b[0m" } else { "" }
+                );
+                stdout.flush().unwrap();
+            }
         }
         else
         {
@@ -517,6 +540,10 @@ fn main()
             {
                 Vec::new()
             }
+            else if lines[i - 1].starts_with('\0')
+            {
+                lines[i - 1][1..].chars().collect::<Vec<char>>()
+            }
             else
             {
                 lines[i - 1].chars().collect::<Vec<char>>()
@@ -533,13 +560,17 @@ fn main()
                 {
                     '\n' | '\x14' | '\x09' | '\x06' =>
                     {
+                        if c != '\x14' && c != '\x06'
+                        {
+                            execute!(stdout, DisableBlinking).unwrap();
+                        }
                         end = start + get_terminal_dimensions().0
                             - if options.prompt { 3 } else { 1 };
                         if end > input.len()
                         {
                             end = input.len()
                         }
-                        if (!options.real_time_output || long || slow)
+                        if (!options.real_time_output || long || (slow && !firstslow))
                             && !input.is_empty()
                             && c != '\x14'
                             && c != '\x06'
@@ -584,20 +615,69 @@ fn main()
                         }
                         break;
                     }
-                    '\x08' =>
+                    '\x03' =>
                     {
-                        //backspace
-                        if placement - start == 0 && start != 0
-                        {
-                            start -= 1;
-                        }
+                        //ctrl+backspace
                         if placement != 0
+                            && matches!(
+                                input[placement - 1],
+                                '(' | '{'
+                                    | '['
+                                    | ')'
+                                    | '}'
+                                    | ']'
+                                    | '+'
+                                    | '-'
+                                    | '*'
+                                    | '/'
+                                    | '^'
+                                    | '<'
+                                    | '='
+                                    | '>'
+                                    | '|'
+                                    | '&'
+                                    | '!'
+                            )
                         {
                             placement -= 1;
                             input.remove(placement);
                         }
-                        end = start + get_terminal_dimensions().0
-                            - if options.prompt { 3 } else { 1 };
+                        else
+                        {
+                            for (i, c) in input[..placement].iter().rev().enumerate()
+                            {
+                                if c.is_whitespace() || i + 1 == placement
+                                {
+                                    input.drain(placement - i - 1..placement);
+                                    placement -= i + 1;
+                                    break;
+                                }
+                                if matches!(
+                                    c,
+                                    '(' | '{'
+                                        | '['
+                                        | ')'
+                                        | '}'
+                                        | ']'
+                                        | '+'
+                                        | '-'
+                                        | '*'
+                                        | '/'
+                                        | '^'
+                                        | '<'
+                                        | '='
+                                        | '>'
+                                        | '|'
+                                        | '&'
+                                        | '!'
+                                )
+                                {
+                                    input.drain(placement - i..placement);
+                                    placement -= i;
+                                    break;
+                                }
+                            }
+                        }
                         if end > input.len()
                         {
                             end = input.len()
@@ -612,6 +692,7 @@ fn main()
                         }
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -657,6 +738,91 @@ fn main()
                         {
                             print!("\x1b[{}D", end - placement)
                         }
+                        if input.is_empty()
+                        {
+                            slow = false;
+                            clear(&input, start, end, options, &colors);
+                        }
+                    }
+                    '\x08' =>
+                    {
+                        //backspace
+                        if placement - start == 0 && start != 0
+                        {
+                            start -= 1;
+                        }
+                        if placement != 0
+                        {
+                            placement -= 1;
+                            input.remove(placement);
+                        }
+                        end = start + get_terminal_dimensions().0
+                            - if options.prompt { 3 } else { 1 };
+                        if end > input.len()
+                        {
+                            end = input.len()
+                        }
+                        if i == lines.len()
+                        {
+                            current.clone_from(&input);
+                        }
+                        else
+                        {
+                            lines[i] = input.clone().iter().collect::<String>();
+                        }
+                        if options.real_time_output && !slow
+                        {
+                            execute!(stdout, DisableBlinking).unwrap();
+                            (frac, graphable, long, varcheck) = print_concurrent(
+                                &input,
+                                &last,
+                                vars.clone(),
+                                options,
+                                colors.clone(),
+                                start,
+                                end,
+                                false,
+                            );
+                            if watch.elapsed().as_millis() > options.slowcheck
+                            {
+                                firstslow = true;
+                                slow = true;
+                            }
+                        }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
+                        else
+                        {
+                            clearln(&input, start, end, options, &colors);
+                        }
+                        if options.debug
+                        {
+                            let time = watch.elapsed().as_nanos();
+                            print!(
+                                " {}\x1b[{}D",
+                                time,
+                                time.to_string().len() + 1 + end - placement
+                            );
+                        }
+                        else if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
+                        if input.is_empty()
+                        {
+                            slow = false;
+                            clear(&input, start, end, options, &colors);
+                        }
                     }
                     '\x7F' =>
                     {
@@ -685,6 +851,7 @@ fn main()
                         }
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -730,6 +897,11 @@ fn main()
                         {
                             print!("\x1b[{}D", end - placement)
                         }
+                        if input.is_empty()
+                        {
+                            slow = false;
+                            clear(&input, start, end, options, &colors);
+                        }
                     }
                     '\x11' =>
                     {
@@ -746,40 +918,7 @@ fn main()
                             input.len()
                                 - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
                         };
-                        if options.real_time_output && !slow
-                        {
-                            (frac, graphable, long, varcheck) = print_concurrent(
-                                &input,
-                                &last,
-                                vars.clone(),
-                                options,
-                                colors.clone(),
-                                start,
-                                end,
-                                false,
-                            );
-                            if watch.elapsed().as_millis() > options.slowcheck
-                            {
-                                firstslow = true;
-                                slow = true;
-                            }
-                        }
-                        else if firstslow
-                        {
-                            firstslow = false;
-                            handle_err(
-                                "too slow, will print on enter",
-                                &input,
-                                options,
-                                &colors,
-                                start,
-                                end,
-                            )
-                        }
-                        else
-                        {
-                            clearln(&input, start, end, options, &colors);
-                        }
+                        clearln(&input, start, end, options, &colors);
                     }
                     '\x18' =>
                     {
@@ -789,6 +928,7 @@ fn main()
                         placement = 0;
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -833,6 +973,7 @@ fn main()
                         end = input.len();
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -879,6 +1020,7 @@ fn main()
                         input.extend(cut);
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -925,6 +1067,7 @@ fn main()
                             input.insert(placement + 1, char);
                             if options.real_time_output && !slow
                             {
+                                execute!(stdout, DisableBlinking).unwrap();
                                 (frac, graphable, long, varcheck) = print_concurrent(
                                     &input,
                                     &last,
@@ -968,6 +1111,7 @@ fn main()
                         //ctrl+l
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -1019,40 +1163,7 @@ fn main()
                         {
                             get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                         };
-                        if options.real_time_output && !slow
-                        {
-                            (frac, graphable, long, varcheck) = print_concurrent(
-                                &input,
-                                &last,
-                                vars.clone(),
-                                options,
-                                colors.clone(),
-                                start,
-                                end,
-                                false,
-                            );
-                            if watch.elapsed().as_millis() > options.slowcheck
-                            {
-                                firstslow = true;
-                                slow = true;
-                            }
-                        }
-                        else if firstslow
-                        {
-                            firstslow = false;
-                            handle_err(
-                                "too slow, will print on enter",
-                                &input,
-                                options,
-                                &colors,
-                                start,
-                                end,
-                            )
-                        }
-                        else
-                        {
-                            clearln(&input, start, end, options, &colors);
-                        }
+                        clearln(&input, start, end, options, &colors);
                         if end - placement != 0
                         {
                             print!("\x1b[{}D", end - placement)
@@ -1061,8 +1172,8 @@ fn main()
                     '\x1D' | '\x05' =>
                     {
                         //up history
-                        i -= if i > 1 { 1 } else { 0 };
-                        if lines.len() == 1
+                        i -= if i > 0 { 1 } else { 0 };
+                        if lines.is_empty()
                         {
                             continue;
                         }
@@ -1100,6 +1211,7 @@ fn main()
                         };
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -1164,6 +1276,7 @@ fn main()
                         };
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -1339,6 +1452,7 @@ fn main()
                         }
                         if options.real_time_output && !slow
                         {
+                            execute!(stdout, DisableBlinking).unwrap();
                             (frac, graphable, long, varcheck) = print_concurrent(
                                 &input,
                                 &last,
@@ -1409,7 +1523,8 @@ fn main()
                 );
             }
             stdout.flush().unwrap();
-            if input.is_empty() && !varcheck
+            execute!(stdout, EnableBlinking).unwrap();
+            if input.is_empty()
             {
                 continue;
             }
@@ -1425,6 +1540,7 @@ fn main()
                 file.as_mut().unwrap(),
                 unmod_lines.as_mut().unwrap(),
                 slow,
+                last.iter().collect::<String>(),
             );
         }
         if varcheck
@@ -1456,7 +1572,7 @@ fn main()
         }
         else if options.graph && graphable
         {
-            let mut inputs: Vec<String> = input
+            let inputs: Vec<String> = input
                 .iter()
                 .collect::<String>()
                 .split('#')
@@ -1464,74 +1580,6 @@ fn main()
                 .collect();
             if inputs.len() < 7
             {
-                let mut funcs = Vec::new();
-                let mut vars = vars.clone();
-                let mut options = options;
-                let mut colors = colors.clone();
-                for (i, s) in inputs.clone().iter().enumerate()
-                {
-                    if s.is_empty()
-                    {
-                        continue;
-                    }
-                    {
-                        options.prec = options.graph_prec;
-                        let split = s.split(|c| c == ';');
-                        let count = split.clone().count();
-                        if count != 1
-                        {
-                            inputs[i] = split.clone().last().unwrap().to_string();
-                            for (i, s) in split.enumerate()
-                            {
-                                if i == count - 1
-                                {
-                                    break;
-                                }
-                                silent_commands(
-                                    &mut options,
-                                    &s.chars()
-                                        .filter(|c| !c.is_whitespace())
-                                        .collect::<Vec<char>>(),
-                                );
-                                if s.contains('=')
-                                {
-                                    if let Err(s) = set_commands_or_vars(
-                                        &mut colors,
-                                        &mut options,
-                                        &mut vars,
-                                        &s.chars().collect::<Vec<char>>(),
-                                    )
-                                    {
-                                        print!(
-                                            "\x1b[G\x1b[A\x1b[K{}\n\x1b[G{}",
-                                            s,
-                                            prompt(options, &colors)
-                                        );
-                                        stdout.flush().unwrap()
-                                    }
-                                }
-                            }
-                        }
-                    }
-                    options.prec = options.graph_prec;
-                    funcs.push(
-                        match input_var(
-                            &inputs[i],
-                            vars.clone(),
-                            &mut Vec::new(),
-                            &mut 0,
-                            options,
-                            false,
-                            false,
-                            0,
-                            Vec::new(),
-                        )
-                        {
-                            Ok(f) => (f.0, f.1, options),
-                            _ => continue 'main,
-                        },
-                    );
-                }
                 let watch = if options.debug
                 {
                     Some(Instant::now())
@@ -1540,7 +1588,31 @@ fn main()
                 {
                     None
                 };
-                handles.push(graph(inputs, funcs, watch, colors.clone()));
+                if options.graph_cli
+                {
+                    if options.interactive
+                    {
+                        terminal::disable_raw_mode().unwrap();
+                    }
+                    graph(inputs, vars.clone(), options, watch, colors.clone(), true)
+                        .join()
+                        .unwrap();
+                    if options.interactive
+                    {
+                        terminal::enable_raw_mode().unwrap();
+                    }
+                }
+                else
+                {
+                    handles.push(graph(
+                        inputs,
+                        vars.clone(),
+                        options,
+                        watch,
+                        colors.clone(),
+                        false,
+                    ));
+                }
             }
         }
     }
