@@ -1,7 +1,11 @@
 use crate::{
-    complex::{NumStr, NumStr::Str},
+    complex::{
+        NumStr,
+        NumStr::{Matrix, Num, Str, Vector},
+    },
     functions::functions,
     math::do_math,
+    print::get_output,
     Colors, Options,
 };
 use crossterm::event::{read, Event, KeyCode, KeyEvent, KeyModifiers};
@@ -195,6 +199,7 @@ pub fn read_single_char() -> char
                 (KeyCode::Esc, _) => digraph(),
                 (KeyCode::Enter, KeyModifiers::NONE) => '\n',
                 (KeyCode::Enter, KeyModifiers::ALT) => '\x09',
+                (KeyCode::Char('h'), KeyModifiers::CONTROL) => '\x03',
                 (KeyCode::Backspace, KeyModifiers::NONE) => '\x08',
                 (KeyCode::Delete, KeyModifiers::NONE) => '\x7F',
                 (KeyCode::Left, KeyModifiers::NONE) => '\x1B',
@@ -217,7 +222,7 @@ pub fn no_col(input: &str, color: bool) -> Vec<char>
         {
             if skip
             {
-                if c == 'm'
+                if c == 'm' || c == 'G' || c == 'K'
                 {
                     skip = false
                 }
@@ -238,9 +243,9 @@ pub fn no_col(input: &str, color: bool) -> Vec<char>
         input.chars().collect::<Vec<char>>()
     }
 }
-pub fn write(mut input: String, file: &mut File, lines: &mut Vec<String>, slow: bool)
+pub fn write(mut input: String, file: &mut File, lines: &mut Vec<String>, slow: bool, last: String)
 {
-    if !lines.is_empty() && lines.last().unwrap() != &input && !input.replace(' ', "").is_empty()
+    if last != input && !input.replace(' ', "").is_empty()
     {
         if slow
         {
@@ -348,6 +353,8 @@ pub fn to_output(input: &[char], color: bool, colors: &Colors) -> String
                         colors.text
                     ))
                 }
+                '@' =>
+                {}
                 _ => output.push(c),
             }
             i += 1;
@@ -422,12 +429,11 @@ pub fn place_funcvarxy(
         {
             for j in i.1.iter_mut()
             {
-                if let Str(s) = j
+                match j
                 {
-                    if matches!(s.as_str(), "x" | "y")
-                    {
-                        *j = num.clone()
-                    }
+                    Str(s) if matches!(s.as_str(), "x" | "y") => *j = num.clone(),
+                    _ =>
+                    {}
                 }
             }
         }
@@ -446,12 +452,11 @@ pub fn place_funcvar(
         {
             for j in i.1.iter_mut()
             {
-                if let Str(s) = j
+                match j
                 {
-                    if s == var
-                    {
-                        *j = num.clone()
-                    }
+                    Str(s) if s == var => *j = num.clone(),
+                    _ =>
+                    {}
                 }
             }
         }
@@ -558,3 +563,89 @@ pub fn do_math_with_var(
         place_funcvar(func_vars, var, num),
     )
 }
+pub fn parsed_to_string(
+    mut input: Vec<NumStr>,
+    func_vars: Vec<(String, Vec<NumStr>)>,
+    options: &Options,
+    colors: &Colors,
+) -> String
+{
+    let mut i = 0;
+    'main: while i < input.len()
+    {
+        if let Str(s) = &input[i]
+        {
+            for v in &func_vars
+            {
+                if *s == v.0 && !v.0.ends_with(')')
+                {
+                    input[i] = Str('('.to_string());
+                    input.splice(i + 1..i + 1, v.1.clone());
+                    input.insert(i + v.1.len() + 1, Str(')'.to_string()));
+                    continue 'main;
+                }
+            }
+        }
+        i += 1;
+    }
+    let mut out = String::new();
+    for i in input
+    {
+        out.push_str(&match i
+        {
+            Num(n) =>
+            {
+                let n = get_output(*options, colors, &n);
+                format!(
+                    "{}{}{}",
+                    n.0,
+                    n.1,
+                    if options.color { "\x1b[0m" } else { "" }
+                )
+            }
+            Vector(n) =>
+            {
+                let mut str = String::new();
+                let mut num;
+                for i in n
+                {
+                    num = get_output(*options, colors, &i);
+                    str.push_str(&format!(
+                        "{}{}{},",
+                        num.0,
+                        num.1,
+                        if options.color { "\x1b[0m" } else { "" }
+                    ));
+                }
+                str.pop();
+                format!("{{{}}}", str)
+            }
+            Matrix(n) =>
+            {
+                let mut str = String::new();
+                let mut num;
+                for i in n
+                {
+                    for j in i
+                    {
+                        num = get_output(*options, colors, &j);
+                        str.push_str(&format!(
+                            "{}{}{},",
+                            num.0,
+                            num.1,
+                            if options.color { "\x1b[0m" } else { "" }
+                        ));
+                    }
+                }
+                str.pop();
+                format!("{{{}}}", str)
+            }
+            Str(n) if n.starts_with('@') && n.contains('(') =>
+            {
+                n.split('(').next().unwrap().replace('@', "")
+            }
+            Str(n) => n.replace('@', ""),
+        })
+    }
+    to_output(&out.chars().collect::<Vec<char>>(), options.color, colors)
+}
