@@ -31,7 +31,7 @@ pub fn input_var(
     //     return Err("debugfail");
     // }
     let mut funcvars = Vec::new();
-    if input.starts_with("history") || input.is_empty()
+    if input.starts_with("history") || input.starts_with("onaxis") || input.is_empty()
     {
         return Err(" ");
     }
@@ -154,10 +154,17 @@ pub fn input_var(
             {
                 pow = '1'.to_string();
             }
-            if !output.is_empty()
-                && (output.last().unwrap().num().is_ok() || output.last().unwrap().str_is(")"))
+            match output.last()
             {
-                output.push(Str('^'.to_string()));
+                Some(Num(_)) => output.push(Str('^'.to_string())),
+                Some(Str(s))
+                    if matches!(s.as_str(), "x" | "y" | "rnd")
+                        || sumrec.iter().any(|v| &v.1 == s) =>
+                {
+                    output.push(Str('^'.to_string()))
+                }
+                _ =>
+                {}
             }
             output.push(Num(match Complex::parse(pow.as_bytes())
             {
@@ -209,11 +216,7 @@ pub fn input_var(
             {
                 num.insert(0, '0')
             }
-            place_multiplier(
-                &mut output,
-                sumrec,
-                if print { Some(vars.clone()) } else { None },
-            );
+            place_multiplier(&mut output, sumrec);
             if neg
             {
                 if chars.len() > i + 1
@@ -228,6 +231,11 @@ pub fn input_var(
                 }
                 neg = false;
             }
+            if num == "0" && i != chars.len() && chars[i] == '⁻'
+            {
+                num.insert(0, '-');
+                i += 1;
+            }
             output.push(Num(Complex::parse(num.clone())
                 .unwrap()
                 .complete(options.prec)));
@@ -281,6 +289,12 @@ pub fn input_var(
                             i += 3;
                             continue;
                         }
+                        if i + 1 < chars.len() && chars[i] == '⁻' && chars[i + 1] == '¹'
+                        {
+                            s.insert(0, 'a');
+                            i += 2;
+                            continue;
+                        }
                         if i + 1 < chars.len()
                             && chars[i] == '^'
                             && (chars[i + 1].is_ascii_digit() || chars[i + 1] == '-')
@@ -376,11 +390,7 @@ pub fn input_var(
                 '{' =>
                 {
                     *bracket += 1;
-                    place_multiplier(
-                        &mut output,
-                        sumrec,
-                        if print { Some(vars.clone()) } else { None },
-                    );
+                    place_multiplier(&mut output, sumrec);
                     output.push(Str("{".to_string()));
                 }
                 '}' =>
@@ -423,6 +433,13 @@ pub fn input_var(
                         output.push(Str("//".to_string()));
                         i += 1;
                     }
+                    else if chars[i + 1] == '-'
+                    {
+                        place_multiplier(&mut output, sumrec);
+                        output.push(Num(n1.clone()));
+                        output.push(Str('/'.to_string()));
+                        i += 1;
+                    }
                     else
                     {
                         output.push(Str('/'.to_string()));
@@ -549,11 +566,7 @@ pub fn input_var(
                     {
                         subfact.1 = *bracket;
                     }
-                    place_multiplier(
-                        &mut output,
-                        sumrec,
-                        if print { Some(vars.clone()) } else { None },
-                    );
+                    place_multiplier(&mut output, sumrec);
                     output.push(Str("(".to_string()))
                 }
                 ')' if i != 0 =>
@@ -583,7 +596,7 @@ pub fn input_var(
                 }
                 '|' =>
                 {
-                    if !abs.is_empty() && abs[0] == *bracket && can_abs(&output)
+                    if !abs.is_empty() && abs[0] == *bracket && can_abs(&output, &vars)
                     {
                         *bracket -= 1;
                         if (i + 2 >= chars.len() || chars[i + 1] != '^') && pwr.1 == *bracket
@@ -604,18 +617,14 @@ pub fn input_var(
                         i += 2;
                         continue;
                     }
-                    else
+                    else if i + 1 != chars.len()
                     {
                         *bracket += 1;
                         if subfact.0
                         {
                             subfact.1 = *bracket;
                         }
-                        place_multiplier(
-                            &mut output,
-                            sumrec,
-                            if print { Some(vars.clone()) } else { None },
-                        );
+                        place_multiplier(&mut output, sumrec);
                         output.push(Str("(".to_string()));
                         output.push(Str("norm".to_string()));
                         output.push(Str("(".to_string()));
@@ -725,11 +734,7 @@ pub fn input_var(
                     }
                     else if i != chars.len() - 1
                         && (chars[i + 1].is_alphanumeric()
-                            || chars[i + 1] == '('
-                            || chars[i + 1] == '{'
-                            || chars[i + 1] == '|'
-                            || chars[i + 1] == '-'
-                            || chars[i + 1] == '!')
+                            || matches!(chars[i + 1], '(' | '{' | '|' | '-' | '!'))
                     {
                         output.push(Str("(".to_string()));
                         output.push(Str("subfact".to_string()));
@@ -796,7 +801,7 @@ pub fn input_var(
             {
                 depthcheck = !depthcheck;
             }
-            else if c.is_alphabetic() || c == &'\'' || c == &'`'
+            else if c.is_alphabetic() || c == &'\'' || c == &'`' || (c == &'2' && word == "atan")
             {
                 word.push(*c);
             }
@@ -893,11 +898,7 @@ pub fn input_var(
                             }
                         }
                     }
-                    place_multiplier(
-                        &mut output,
-                        sumrec,
-                        if print { Some(vars.clone()) } else { None },
-                    );
+                    place_multiplier(&mut output, sumrec);
                     output.push(Str(word.clone()));
                     output.push(Str("(".to_string()));
                     if sumrec.iter().any(|c| c.0 == -1)
@@ -932,15 +933,14 @@ pub fn input_var(
             && i + countv < chars.len()
             && matches!(
                 chars[i + countv],
-                'x' | 'y' | 'z' | '(' | '|' | '{' | '0'..='9' | '^'
+                'x' | 'y' | 'z' | '(' | '|' | '{' | '0'..='9' | '^' | '⁻'
+            ))
+            || matches!(
+                word.as_str(),
+                "rnd" | "inf" | "true" | "false" | "nan" | "NaN"
             ))
-            || matches!(word.as_str(), "rnd" | "inf" | "nan" | "NaN"))
         {
-            place_multiplier(
-                &mut output,
-                sumrec,
-                if print { Some(vars.clone()) } else { None },
-            );
+            place_multiplier(&mut output, sumrec);
             if neg
             {
                 output.push(Num(n1.clone()));
@@ -948,12 +948,20 @@ pub fn input_var(
                 neg = false;
             }
             i += countv;
-            if matches!(word.as_str(), "inf" | "nan" | "NaN")
+            if matches!(word.as_str(), "inf" | "nan" | "NaN" | "true" | "false")
             {
                 if matches!(word.as_str(), "nan" | "NaN")
                 {
                     output.push(Num(Complex::with_val(options.prec, Nan)));
                 }
+                else if word == "true"
+                {
+                    output.push(Num(Complex::with_val(options.prec, 1)));
+                }
+                else if word == "false"
+                {
+                    output.push(Num(Complex::new(options.prec)));
+                }
                 else
                 {
                     output.push(Num(Complex::with_val(options.prec, Infinity)));
@@ -1011,11 +1019,7 @@ pub fn input_var(
                 }
             }))
         {
-            place_multiplier(
-                &mut output,
-                sumrec,
-                if print { Some(vars.clone()) } else { None },
-            );
+            place_multiplier(&mut output, sumrec);
             if neg
             {
                 output.push(Num(n1.clone()));
@@ -1071,21 +1075,37 @@ pub fn input_var(
                             && (chars[i + 1].is_numeric() || chars[i + 1] == '-')))
                 {
                     let j = i;
-                    let vn = var.name.split(|c| matches!(c, '(' | '{')).next().unwrap();
+                    let vn = var
+                        .name
+                        .split(|c| matches!(c, '(' | '{' | '[' | '|'))
+                        .next()
+                        .unwrap();
                     if var.name.contains(&'(')
                         && i + vn.len() < chars.len()
                         && chars[i..i + vn.len()] == *vn
-                        && matches!(chars[i + vn.len()], '(' | '{')
+                        && matches!(chars[i + vn.len()], '(' | '{' | '[' | '|')
                     {
+                        let abs = chars[i + vn.len()] == '|';
                         let countj = vn.len();
                         let mut count = 0;
+                        let mut abstest = false;
                         for (f, c) in chars[i..].iter().enumerate()
                         {
-                            if *c == '('
+                            if abs && *c == '|' && (count == 0 || (abstest && count == 1))
+                            {
+                                if abstest
+                                {
+                                    i += f;
+                                    break;
+                                }
+                                count += 1;
+                                abstest = true
+                            }
+                            else if matches!(c, '(' | '{' | '[')
                             {
                                 count += 1;
                             }
-                            else if *c == ')'
+                            else if matches!(c, ')' | '}' | ']')
                             {
                                 count -= 1;
                                 if count == 0
@@ -1107,15 +1127,24 @@ pub fn input_var(
                         let mut ccount = 0;
                         for c in &chars[j..i]
                         {
-                            if *c == ',' && count == if chars[j + 1] == '{' { 0 } else { 1 }
+                            if *c == ','
+                                && count
+                                    == if matches!(chars[j + 1], '{' | '[')
+                                    {
+                                        0
+                                    }
+                                    else
+                                    {
+                                        1
+                                    }
                             {
                                 ccount += 1;
                             }
-                            else if *c == '(' || c == &'{'
+                            else if matches!(c, '(' | '{' | '[')
                             {
                                 count += 1;
                             }
-                            else if *c == ')' || c == &'}' || c == &']'
+                            else if matches!(c, ')' | '}' | ']')
                             {
                                 count -= 1;
                             }
@@ -1133,11 +1162,7 @@ pub fn input_var(
                         }
                         if var.name.contains(&',') && chars.len() > 4
                         {
-                            place_multiplier(
-                                &mut output,
-                                sumrec,
-                                if print { Some(vars.clone()) } else { None },
-                            );
+                            place_multiplier(&mut output, sumrec);
                             if neg
                             {
                                 output.push(Num(n1.clone()));
@@ -1161,11 +1186,11 @@ pub fn input_var(
                             count = 0;
                             for (f, c) in temp.iter().enumerate()
                             {
-                                if c == &'(' || c == &'{'
+                                if matches!(c, '(' | '{' | '[')
                                 {
                                     count += 1;
                                 }
-                                else if c == &')' || c == &'}' || c == &']'
+                                else if matches!(c, ')' | '}' | ']')
                                 {
                                     count -= 1;
                                 }
@@ -1186,7 +1211,7 @@ pub fn input_var(
                             start = 0;
                             for (f, c) in var.name.iter().enumerate()
                             {
-                                if c == &'(' || c == &'{'
+                                if matches!(c, '(' | '{' | '[')
                                 {
                                     if count == 0
                                     {
@@ -1194,7 +1219,7 @@ pub fn input_var(
                                     }
                                     count += 1;
                                 }
-                                else if c == &')' || c == &'}' || c == &']'
+                                else if matches!(c, ')' | '}' | ']')
                                 {
                                     count -= 1;
                                     if count == 0
@@ -1422,11 +1447,7 @@ pub fn input_var(
                         }
                         else
                         {
-                            place_multiplier(
-                                &mut output,
-                                sumrec,
-                                if print { Some(vars.clone()) } else { None },
-                            );
+                            place_multiplier(&mut output, sumrec);
                             if neg
                             {
                                 output.push(Num(n1.clone()));
@@ -1529,6 +1550,14 @@ pub fn input_var(
                                     }]
                                 }
                             };
+                            if abs
+                            {
+                                num.insert(0, Str("(".to_string()));
+                                num.insert(0, Str("norm".to_string()));
+                                num.insert(0, Str("(".to_string()));
+                                num.push(Str(")".to_string()));
+                                num.push(Str(")".to_string()))
+                            }
                             if print && num.len() == 1
                             {
                                 num.insert(0, Str("(".to_string()));
@@ -1732,11 +1761,7 @@ pub fn input_var(
                         {
                             var.name.len()
                         };
-                        place_multiplier(
-                            &mut output,
-                            sumrec,
-                            if print { Some(vars.clone()) } else { None },
-                        );
+                        place_multiplier(&mut output, sumrec);
                         if neg
                         {
                             output.push(Num(n1.clone()));
@@ -1822,11 +1847,7 @@ pub fn input_var(
                                 }
                             }
                         }
-                        place_multiplier(
-                            &mut output,
-                            sumrec,
-                            if print { Some(vars.clone()) } else { None },
-                        );
+                        place_multiplier(&mut output, sumrec);
                         output.push(Num(Complex::with_val(options.prec, 10)));
                         if i + 1 != chars.len()
                             && (chars[i + 1].is_alphanumeric()
@@ -1843,11 +1864,7 @@ pub fn input_var(
                     'x' | 'y' =>
                     {
                         graph = true;
-                        place_multiplier(
-                            &mut output,
-                            sumrec,
-                            if print { Some(vars.clone()) } else { None },
-                        );
+                        place_multiplier(&mut output, sumrec);
                         output.push(Str(c.to_string()));
                         if scientific
                         {
@@ -1874,11 +1891,7 @@ pub fn input_var(
                     }
                     'i' =>
                     {
-                        place_multiplier(
-                            &mut output,
-                            sumrec,
-                            if print { Some(vars.clone()) } else { None },
-                        );
+                        place_multiplier(&mut output, sumrec);
                         output.push(Num(Complex::with_val(options.prec, (0, 1))));
                         if pwr.0
                             && pwr.1 == *bracket
@@ -1906,11 +1919,7 @@ pub fn input_var(
                     'z' =>
                     {
                         graph = true;
-                        place_multiplier(
-                            &mut output,
-                            sumrec,
-                            if print { Some(vars.clone()) } else { None },
-                        );
+                        place_multiplier(&mut output, sumrec);
                         output.push(Str('('.to_string()));
                         output.push(Str('x'.to_string()));
                         output.push(Str('+'.to_string()));
@@ -1960,10 +1969,16 @@ pub fn input_var(
         {
             pow = "1".to_string();
         }
-        if !output.is_empty()
-            && (output.last().unwrap().num().is_ok() || output.last().unwrap().str_is(")"))
+        match output.last()
         {
-            output.push(Str("^".to_string()));
+            Some(Num(_)) => output.push(Str('^'.to_string())),
+            Some(Str(s))
+                if matches!(s.as_str(), "x" | "y" | "rnd") || sumrec.iter().any(|v| &v.1 == s) =>
+            {
+                output.push(Str('^'.to_string()))
+            }
+            _ =>
+            {}
         }
         output.push(Num(Complex::with_val(
             options.prec,
@@ -2037,47 +2052,6 @@ pub fn input_var(
         }
         i += 1;
     }
-    while let Some(Str(c)) = output.last()
-    {
-        if print && vars.iter().any(|a| a.name.iter().collect::<String>() == *c)
-        {
-            break;
-        }
-        else if !matches!(c.as_str(), "rnd" | ")" | "}" | "x" | "y")
-            && !sumrec.iter().any(|s| &s.1 == c)
-            && !c.starts_with('@')
-        {
-            output.pop();
-        }
-        else if output.len() > 1
-            && output[output.len() - 2].str_is("norm")
-            && output[output.len() - 1].str_is(")")
-        {
-            output.pop();
-            output.pop();
-            output.pop();
-        }
-        else
-        {
-            break;
-        }
-    }
-    i = 1;
-    while i + 1 < output.len()
-    {
-        if Str("*".to_string()) == output[i]
-        {
-            if let Str(c) = &output[i + 1]
-            {
-                if matches!(c.as_str(), "+" | "-" | "*" | "/" | "//" | "^" | "^^" | "%")
-                {
-                    output.remove(i);
-                    continue;
-                }
-            }
-        }
-        i += 1;
-    }
     if !err.is_empty()
     {
         return Err(err);
@@ -2094,7 +2068,7 @@ pub fn input_var(
             {
                 match s.as_str()
                 {
-                    "x" | "y" =>
+                    "x" | "y" | "roll" | "rnd" =>
                     {
                         to = 0;
                     }
@@ -2128,40 +2102,32 @@ pub fn input_var(
     }
     Ok((output, funcvars, graph, false))
 }
-fn place_multiplier(
-    output: &mut Vec<NumStr>,
-    sumrec: &[(isize, String)],
-    vars: Option<Vec<Variable>>,
-)
+fn place_multiplier(output: &mut Vec<NumStr>, sumrec: &[(isize, String)])
 {
-    if let Some(Str(s)) = output.last()
+    match output.last()
     {
-        if matches!(s.as_str(), ")" | "x" | "y" | "]" | "}" | "rnd" | "@")
-            || sumrec
-                .iter()
-                .any(|a| a.1 == *s || "@".to_owned() + &a.1 == *s)
-            || (if let Some(n) = vars
-            {
-                n.iter().any(|a| a.name.iter().collect::<String>() == *s)
-            }
-            else
-            {
-                false
-            })
+        Some(Str(s))
+            if matches!(s.as_str(), ")" | "x" | "y" | "]" | "}" | "rnd" | "@")
+                || sumrec
+                    .iter()
+                    .any(|a| a.1 == *s || "@".to_owned() + &a.1 == *s) =>
         {
             output.push(Str('*'.to_string()))
         }
-    }
-    else if let Num(_) = output.last().unwrap_or(&Str(String::new()))
-    {
-        output.push(Str('*'.to_string()))
+        Some(Num(_)) => output.push(Str('*'.to_string())),
+        _ =>
+        {}
     }
 }
-fn can_abs(output: &[NumStr]) -> bool
+fn can_abs(output: &[NumStr], vars: &[Variable]) -> bool
 {
     if let Some(Str(s)) = output.last()
     {
-        return !functions().contains(s.as_str());
+        !(functions().contains(s.as_str())
+            || vars.iter().any(|c| c.name.iter().collect::<String>() == *s))
+    }
+    else
+    {
+        true
     }
-    true
 }
